@@ -16,10 +16,8 @@ import math
 import libxml2
 from rteval.Log import Log
 from rteval.modules import rtevalModulePrototype
-from rteval.systopology import cpuinfo
-from rteval.systopology import CpuList, SysTopology, collapse_cpulist
-
-expand_cpulist = CpuList.expand_cpulist
+from rteval.systopology import cpuinfo, SysTopology
+from rteval.cpulist_utils import expand_cpulist, collapse_cpulist
 
 class RunData:
     '''class to keep instance data from a cyclictest run'''
@@ -27,7 +25,7 @@ class RunData:
         self.__id = coreid
         self.__type = datatype
         self.__priority = int(priority)
-        self.__description = ''
+        self.description = ''
         # histogram of data
         self.__samples = {}
         self.__numsamples = 0
@@ -63,12 +61,6 @@ class RunData:
         if value < self.__min:
             self.__min = value
 
-    def sample(self, value):
-        self.__samples[value] += self.__samples.setdefault(value, 0) + 1
-        self.update_max(value)
-        self.update_min(value)
-        self.__numsamples += 1
-
     def bucket(self, index, value):
         self.__samples[index] = self.__samples.setdefault(index, 0) + value
         if value:
@@ -88,26 +80,34 @@ class RunData:
             return
 
         self._log(Log.INFO, f"reducing {self.__id}")
-        total = 0
+        total = 0 # total number of samples
+        total_us = 0
         keys = list(self.__samples.keys())
         keys.sort()
 
-        mid = self.__numsamples / 2
+        # if numsamples is odd, then + 1 gives us the actual mid
+        # if numsamples is even, we avg mid and mid + 1, so we actually
+        # want to know mid + 1 since we will combine it with mid and
+        # the lastkey if the last key is at the end of a previous bucket
+        mid = int(self.__numsamples / 2) + 1
 
         # mean, mode, and median
         occurances = 0
         lastkey = -1
         for i in keys:
-            if mid > total and mid <= (total + self.__samples[i]):
-                if self.__numsamples & 1 and mid == total+1:
+            if mid > total and mid <= total + self.__samples[i]:
+                # Test if numsamples is even and if mid+1 is the next bucket
+                if self.__numsamples & 1 != 0 and mid == total+1:
                     self.__median = (lastkey + i) / 2
                 else:
                     self.__median = i
-            total += (i * self.__samples[i])
+            lastkey = i
+            total += self.__samples[i]
+            total_us += (i * self.__samples[i])
             if self.__samples[i] > occurances:
                 occurances = self.__samples[i]
                 self.__mode = i
-        self.__mean = float(total) / float(self.__numsamples)
+        self.__mean = float(total_us) / float(self.__numsamples)
 
         # range
         for i in keys:
@@ -115,7 +115,7 @@ class RunData:
                 low = i
                 break
         high = keys[-1]
-        while high and self.__samples[high] == 0:
+        while high and self.__samples.get(high, 0) == 0:
             high -= 1
         self.__range = high - low
 
@@ -132,7 +132,7 @@ class RunData:
     def MakeReport(self):
         rep_n = libxml2.newNode(self.__type)
         if self.__type == 'system':
-            rep_n.newProp('description', self.__description)
+            rep_n.newProp('description', self.description)
         else:
             rep_n.newProp('id', str(self.__id))
             rep_n.newProp('priority', str(self.__priority))
@@ -190,41 +190,12 @@ class Cyclictest(rtevalModulePrototype):
         # Create a RunData object per CPU core
         self.__numanodes = int(self.__cfg.setdefault('numanodes', 0))
         self.__priority = int(self.__cfg.setdefault('priority', 95))
-        self.__buckets = int(self.__cfg.setdefault('buckets', 2000))
+        default_buckets = ModuleParameters()["buckets"]["default"]
+        self.__buckets = int(self.__cfg.setdefault('buckets', default_buckets))
         self.__numcores = 0
-        self.__cpus = []
         self.__cyclicdata = {}
-        self.__sparse = False
-        self.__run_on_isolcpus = bool(self.__cfg.setdefault('run-on-isolcpus', False))
-
-        if self.__cfg.cpulist:
-            self.__cpulist = self.__cfg.cpulist
-            self.__cpus = expand_cpulist(self.__cpulist)
-            # Only include online cpus
-            self.__cpus = CpuList(self.__cpus).cpulist
-            # Reset cpulist from the newly calculated self.__cpus
-            self.__cpulist = collapse_cpulist(self.__cpus)
-            self.__cpus = [str(c) for c in self.__cpus]
-            self.__sparse = True
-            if self.__run_on_isolcpus:
-                self._log(Log.WARN, "ignoring --measurement-run-on-isolcpus, since cpulist is specified")
-        else:
-            self.__cpus = SysTopology().online_cpus_str()
-            # Get the cpuset from the environment
-            cpuset = os.sched_getaffinity(0)
-            # Convert the elements to strings
-            cpuset = [str(c) for c in cpuset]
-            # Get isolated CPU list
-            isolcpus = [str(c) for c in SysTopology().isolated_cpus()]
-            # Only include cpus that are in the cpuset and isolated CPUs if run_on_isolcpus is enabled
-            self.__cpus = [c for c in self.__cpus if c in cpuset or self.__run_on_isolcpus and c in isolcpus]
-            if self.__run_on_isolcpus:
-                self.__sparse = True
-                self.__cpulist = collapse_cpulist(self.__cpus)
-
-        # Sort the list of cpus to align with the order reported by cyclictest
-        self.__cpus.sort(key=int)
-
+        self.__cpulist = self.__cfg.setdefault('cpulist', "")
+        self.__cpus = [str(c) for c in expand_cpulist(self.__cpulist)]
         self.__numcores = len(self.__cpus)
 
         info = cpuinfo()
@@ -241,10 +212,7 @@ class Cyclictest(rtevalModulePrototype):
                                               logfnc=self._log)
         self.__cyclicdata['system'].description = (f"({self.__numcores} cores) ") + info['0']['model name']
 
-        if self.__sparse:
-            self._log(Log.DEBUG, f"system using {self.__numcores} cpu cores")
-        else:
-            self._log(Log.DEBUG, f"system has {self.__numcores} cpu cores")
+        self._log(Log.DEBUG, f"system using {self.__numcores} cpu cores")
         self.__started = False
         self.__cyclicoutput = None
         self.__breaktraceval = None
@@ -279,12 +247,8 @@ class Cyclictest(rtevalModulePrototype):
                       f'-h {self.__buckets}',
                       f"-p{int(self.__priority)}",
                       ]
-        if self.__sparse:
-            self.__cmd.append(f'-t{self.__numcores}')
-            self.__cmd.append(f'-a{self.__cpulist}')
-        else:
-            self.__cmd.append('-t')
-            self.__cmd.append('-a')
+        self.__cmd.append(f'-t{self.__numcores}')
+        self.__cmd.append(f'-a{self.__cpulist}')
 
         if 'threads' in self.__cfg and self.__cfg.threads:
             self.__cmd.append(f"-t{int(self.__cfg.threads)}")
@@ -430,14 +394,13 @@ class Cyclictest(rtevalModulePrototype):
         return rep_n
 
 
-
 def ModuleInfo():
     return {"parallel": True,
             "loads": True}
 
 
-
 def ModuleParameters():
+    """ default parameters """
     return {"interval": {"descr": "Base interval of the threads in microseconds",
                          "default": 100,
                          "metavar": "INTV_US"},
@@ -458,6 +421,7 @@ def ModuleParameters():
 
 
 def create(params, logger):
+    """ Instantiate a Cyclictest measurement module object """
     return Cyclictest(params, logger)
 
 
@@ -475,9 +439,7 @@ if __name__ == '__main__':
     cfg.AppendConfig('cyclictest', prms)
 
     cfg_ct = cfg.GetSection('cyclictest')
-    cfg_ct.reportdir = "."
-    cfg_ct.buckets = 200
-    # cfg_ct.breaktrace = 30
+    cfg_ct.cpulist = collapse_cpulist(SysTopology().online_cpus())
 
     runtime = 10
 
